#include "timeractor.h"
#include <QDebug>

TimerActor::TimerActor(const int timeoutSec, QObject *parent): Actor(parent), timeoutMsec_(timeoutSec*1000)
{
    connect(&timer, &QTimer::timeout, this, &TimerActor::timeout);
    timer.setSingleShot(true);
}

void TimerActor::onValueChanged(uint8_t state)
{
    if((state && !triggerValue) || (!state && triggerValue))
    {
        if(timer.isActive()) timer.stop();
        timer.setInterval(timeoutMsec_);
        timer.start();
    }
}

void TimerActor::store(QJsonObject& json)
{
    json["Type"] = "Timer";
    Actor::store(json);
    json["Timeout"] = timeoutMsec_;
}

void TimerActor::load(const QJsonObject& json)
{
    Actor::load(json);
    timeoutMsec_ = json["Timeout"].toInt(10000);
}

void TimerActor::store(QString subsecton, QSettings* settings)
{
    settings->setValue(subsecton + "Type", "Timer");
    Actor::store(subsecton, settings);
    settings->setValue(subsecton + "Timeout", timeoutMsec_);
}
void TimerActor::load(QString subsecton, QSettings* settings)
{
    Actor::load(subsecton, settings);
    timeoutMsec_ = settings->value(subsecton + "Timeout").toInt();
}

void TimerActor::setTimeout(const int timeoutSec)
{
    timeoutMsec_ = timeoutSec*1000;
}

int TimerActor::getTimeout()
{
    return timeoutMsec_*1000;
}

void TimerActor::timeout()
{
    performAction();
}

QString TimerActor::getName()
{
    if(name.size() > 0) return name;
    else
    {
        QString string;
        string = "Timeout after " + QString::number(timeoutMsec_/1000) + " seconds. ";
        return string;
    }
}
