#include "itemstore.h"
#include "relay.h"
#include <QJsonArray>

ItemStore::ItemStore(SensorStore* sensors, QObject *parent): QObject(parent), sensors_(sensors)
{
}

void ItemStore::addItem(std::shared_ptr<Item> item)
{
    bool mached = false;
    for(unsigned i = 0; i < items_.size(); i++ ) if(*items_[i] == *item) mached = true;
    if(!mached)
    {
        items_.push_back(std::shared_ptr<Item>(item));
        items_.back()->setSensorStore(sensors_);
        itemAdded(std::weak_ptr<Item>(items_.back()));
        qDebug()<<"item added";
    }
}

void ItemStore::addItems(const std::vector<std::shared_ptr<Item>>& itemIn)
{
    for(unsigned i = 0; i < items_.size(); i++ )
    {
        if(Relay* relay = dynamic_cast<Relay*>(items_[i].get()))
        {
            bool mached = false;
            for(unsigned j = 0; j < itemIn.size(); j++) if(*(items_[i]) == *(itemIn[j]))
            {
                mached = true;
                if(itemIn[j]->getValue() != items_[i]->getValue()) items_[i]->informValue(itemIn[j]->getValue());
                Relay* relayIn = dynamic_cast<Relay*>(itemIn[j].get());
                if(relayIn)
                {
                    if(relay->getId() != relayIn->getId()) relay->setId(relayIn->getId());
                }

            }
            if(!mached)
            {
                itemDeleted(*items_[i].get());
                items_.erase(items_.begin()+i);
            }
        }
    }

    for(unsigned j = 0; j < itemIn.size(); j++)addItem(itemIn[j]);

}

void ItemStore::itemStateChanged(const ItemData& item)
{
     for(unsigned i = 0; i < items_.size(); i++ )
     {
         if(items_[i]->operator==(item))
         {
            qDebug()<<"is item "<<i<<" with ids: "<<item.id()<<"  "<<items_[i]->id()<<"\nHas state: "<<items_[i]->getValue()<<" wants state: "<<item.getValue();
            if(items_[i]->getValue() != item.getValue())items_[i]->informValue(item.getValue());
         }

     }

}

void ItemStore::store(QJsonObject& json)
{
    QJsonArray itemsArray;
    for(size_t i = 0; i < items_.size(); ++i)
    {
        QJsonObject itemObject;
        items_[i]->store(itemObject);
        itemsArray.append(itemObject);
    }
    json["Items"] = itemsArray;
}

void ItemStore::load(const QJsonObject& json, Microcontroller * const micro)
{
    const QJsonArray itemsArray(json["Items"].toArray(QJsonArray()));
    for(int i = 0; i < itemsArray.size(); ++i)
    {
        if(itemsArray[i].isObject())
        {
            const QJsonObject itemObject = itemsArray[i].toObject();
            if(!itemObject["Name"].isObject()) qDebug()<<"no name";
            std::shared_ptr<Relay> newItem;
            if(itemObject["Type"].toString("") == "Relay")
            {
                newItem = std::shared_ptr<Relay>(new Relay(sensors_, micro));
            }
            else if(itemObject["Type"].toString("") == "Aux")
            {
            }
            if(newItem)
            {
                newItem->load(itemObject);
                addItem(newItem);
            }
        }
    }
}

void ItemStore::store(QString subsecton, QSettings* settings)
{
    settings->setValue(subsecton + "/NumberOfItems", static_cast<unsigned>(items_.size()));
    for(size_t i = 0; i < items_.size(); ++i)
    {
        items_[i]->store(subsecton + "/Item" + QString::number(i), settings);
    }
}

void ItemStore::load(QString subsecton, QSettings* settings, Microcontroller* micro)
{
    unsigned itemLen = settings->value(subsecton + "/NumberOfItems").toUInt();
    for(size_t i = 0; i < itemLen; ++i)
    {
        std::shared_ptr<Relay> newItem;
        if(settings->value(subsecton + "/Item" + QString::number(i)+ "Type").toString() == "Relay")
        {
            newItem = std::shared_ptr<Relay>(new Relay(sensors_, micro));
        }
        else if(settings->value(subsecton + "/Item" + QString::number(i)+ "Type").toString() == "Aux")
        {
        }
        if(newItem)
        {
            newItem->load(subsecton + "/Item" + QString::number(i), settings);
            addItem(newItem);
        }
    }
}
