#include "sensorlistwidget.h"

#include <QDebug>
#include <QHeaderView>

SensorListWidget::SensorListWidget(QWidget *parent): QTableWidget(parent)
{
    setColumnCount(2);
    setSelectionBehavior(QAbstractItemView::SelectRows);
    horizontalHeader()->setSectionResizeMode(0, QHeaderView::Stretch);

    setHorizontalHeaderItem(0, new QTableWidgetItem("Sensor"));
    setHorizontalHeaderItem(1, new QTableWidgetItem("Value"));
}

SensorListWidget::SensorListWidget(SensorStore& sensorStore, QWidget* parent): QTableWidget (parent)
{
      sensorsChanged(*(sensorStore.getSensors()));
}

void SensorListWidget::sensorsChanged(std::vector<Sensor> sensors)
{
    clear();
    setHorizontalHeaderItem(0, new QTableWidgetItem("Sensor"));
    setHorizontalHeaderItem(1, new QTableWidgetItem("Value"));
    setRowCount(sensors.size());
    for(size_t i = 0; i < sensors.size(); ++i)
    {
        QString itemString;
        itemString.append(QString::number(sensors[i].field));
        itemString.append(' ');

        if(sensors[i].type == Sensor::TYPE_DOOR)
        {
            if(sensors[i].field) itemString.append("\"Open\"");
            else itemString.append("\"Closed\"");
        }
        else if(sensors[i].type == Sensor::TYPE_AUDIO_OUTPUT)
        {
            if(sensors[i].field) itemString.append("\"Playing\"");
            else itemString.append("\"Silent\"");
        }
        setItem(i, 0, new QTableWidgetItem(sensors[i].name));
        setItem(i, 1, new QTableWidgetItem(itemString));
    }
}

